"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _iterateJsdoc = _interopRequireDefault(require("../iterateJsdoc"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _default = (0, _iterateJsdoc.default)(({
  jsdoc,
  report,
  node,
  utils
}) => {
  // Implicit return like `() => foo` is ok
  if (node.type === 'ArrowFunctionExpression' && node.expression) {
    return;
  } // Async function always returns a promise


  if (node.async) {
    return;
  }

  const targetTagName = utils.getPreferredTagName('returns'); // We can skip in case there are no tags defined...

  if (typeof jsdoc.tags === 'undefined') {
    return;
  }

  const jsdocTags = jsdoc.tags.filter(item => {
    return item.tag === targetTagName;
  });

  if (jsdocTags.length === 0) {
    return;
  }

  if (jsdocTags.length > 1) {
    report('Found more than one @' + targetTagName + ' declaration.');
    return;
  }

  const returnsTagType = jsdocTags[0].type && jsdocTags[0].type.trim();

  if (returnsTagType === 'void' || returnsTagType === 'undefined') {
    return;
  } // An abstract function is by definition incomplete
  // so it is perfectly fine if the return is missing
  // a subclass may inherits the doc an implements the
  // missing return.


  const isAbstract = jsdoc.tags.some(item => {
    return item.tag === utils.getPreferredTagName('abstract');
  });

  if (isAbstract) {
    return;
  }

  const sourcecode = utils.getFunctionSourceCode();

  if (sourcecode.indexOf('return') === -1) {
    report('Present JSDoc @' + targetTagName + ' declaration but not available return expression in function.');
  }
});

exports.default = _default;
module.exports = exports.default;
//# sourceMappingURL=requireReturnsCheck.js.map